/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "inputmethodaddon.h"

Fcitx5InputMethodAddon::Fcitx5InputMethodAddon(fcitx::Instance *instance)
    : instance_(instance) {
    engineManager_ = std::make_shared<EngineManager>(instance_);
    uiProxy_ = std::make_unique<UIProxy>(instance_, engineManager_.get());
}

Fcitx5InputMethodAddon::~Fcitx5InputMethodAddon() = default;

void Fcitx5InputMethodAddon::activate(const fcitx::InputMethodEntry &entry,
                                      fcitx::InputContextEvent &event) {
    uiProxy_->notifyIMEActivate();
    engineManager_->currentInputMethodEngine().activate(entry, event);
}

void Fcitx5InputMethodAddon::deactivate(const fcitx::InputMethodEntry &entry,
                                        fcitx::InputContextEvent &event) {
    uiProxy_->notifyIMEDeactivate();
    engineManager_->currentInputMethodEngine().deactivate(entry, event);
}

void Fcitx5InputMethodAddon::keyEvent(const fcitx::InputMethodEntry &entry,
                                      fcitx::KeyEvent &keyEvent) {
    engineManager_->currentInputMethodEngine().keyEvent(entry, keyEvent);
    auto *inputContext = keyEvent.inputContext();
    if (inputContext == nullptr) {
        return;
    }
    inputContext->inputPanel().setCustomInputPanelCallback(
        [this](fcitx::InputContext *inputContext) {
            uiProxy_->updateInputPanel(inputContext);
        });
}

void Fcitx5InputMethodAddon::reloadConfig() {
    engineManager_->currentInputMethodEngine().reloadConfig();
}

void Fcitx5InputMethodAddon::reset(const fcitx::InputMethodEntry &entry,
                                   fcitx::InputContextEvent &event) {
    engineManager_->currentInputMethodEngine().reset(entry, event);
}

void Fcitx5InputMethodAddon::save() {
    engineManager_->currentInputMethodEngine().save();
}

std::string
Fcitx5InputMethodAddon::subMode(const fcitx::InputMethodEntry &entry,
                                fcitx::InputContext &inputContext) {
    return engineManager_->currentInputMethodEngine().subMode(entry,
                                                              inputContext);
}

void Fcitx5InputMethodAddon::invokeActionImpl(
    const fcitx::InputMethodEntry &entry, fcitx::InvokeActionEvent &event) {
    engineManager_->currentInputMethodEngine().invokeActionImpl(entry, event);
}

const fcitx::Configuration *Fcitx5InputMethodAddon::getConfig() const {
    return engineManager_->currentInputMethodEngine().getConfig();
}

void Fcitx5InputMethodAddon::setConfig(const fcitx::RawConfig &config) {
    engineManager_->currentInputMethodEngine().setConfig(config);
}

void Fcitx5InputMethodAddon::setSubConfig(const std::string &path,
                                          const fcitx::RawConfig &config) {
    engineManager_->currentInputMethodEngine().setSubConfig(path, config);
}

FCITX_ADDON_FACTORY(Fcitx5InputMethodAddonFactory)