/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "inputwindowproxy.h"

static const char uiServiceName_[] = "org.ukui.okinputmethod";
static const char uiServicePatch_[] = "/org/ukui/okinputmethod";
static const char uiServiceInterface_[] = "org.ukui.okinputmethod";

static const char inputWindowBackendName_[] = "org.ukui.inputmethodbackend";
static const char inputWindowBackendPatch_[] = "/inputwindowproxy";
static const char inputWindowBackendInterface_[] =
    "org.ukui.inputmethod.inputwindowproxy";

using GetCandidateListSizeCallback = std::function<int()>;
using GetCandidateWordCallback =
    std::function<const fcitx::CandidateWord &(int idx)>;
static inline const fcitx::CandidateWord *nthCandidateIgnorePlaceholder(
    GetCandidateListSizeCallback getCandidateListSizeCallback,
    GetCandidateWordCallback getCandidateWordCallback, int idx) {
    int total = 0;
    const int size = getCandidateListSizeCallback();
    if (idx < 0 || idx >= size) {
        return nullptr;
    }
    for (int i = 0; i < size; i++) {
        const auto &candidate = getCandidateWordCallback(i);
        if (candidate.isPlaceHolder()) {
            continue;
        }
        if (idx == total) {
            return &candidate;
        }
        ++total;
    }
    return nullptr;
}

static inline const fcitx::CandidateWord *
nthCandidateIgnorePlaceholder(const fcitx::CandidateList &candidateList,
                              int idx) {
    return nthCandidateIgnorePlaceholder(
        [&candidateList]() { return candidateList.size(); },
        [&candidateList](int idx) -> const fcitx::CandidateWord & {
            return candidateList.candidate(idx);
        },
        idx);
}

static inline const fcitx::CandidateWord *nthBulkCandidateIgnorePlaceholder(
    const fcitx::BulkCandidateList &bulkCandidateList, int idx) {

    return nthCandidateIgnorePlaceholder(
        [&bulkCandidateList]() { return bulkCandidateList.totalSize(); },
        [&bulkCandidateList](int idx) -> const fcitx::CandidateWord & {
            return bulkCandidateList.candidateFromAll(idx);
        },
        idx);
}

class InputWindowBackend
    : public fcitx::dbus::ObjectVTable<InputWindowBackend> {
public:
    explicit InputWindowBackend(InputWindowProxy *parent) : parent_(parent) {}

    ~InputWindowBackend() = default;

    void selectCandidate(int index) { parent_->selectCandidate(index); }
    void prevPage() { parent_->prevPage(); }
    void nextPage() { parent_->nextPage(); }

private:
    FCITX_OBJECT_VTABLE_METHOD(selectCandidate, "SelectCandidate", "i", "");
    FCITX_OBJECT_VTABLE_METHOD(prevPage, "PrevPage", "", "");
    FCITX_OBJECT_VTABLE_METHOD(nextPage, "NextPage", "", "");

private:
    InputWindowProxy *parent_;
};

InputWindowProxy::InputWindowProxy(fcitx::Instance *instance,
                                   fcitx::dbus::Bus *bus)
    : instance_(instance), bus_(bus) {
    initInputWindowBackend();
}

InputWindowProxy::~InputWindowProxy() {
    bus_->releaseName(inputWindowBackendName_);
}

void InputWindowProxy::initInputWindowBackend() {
    inputWindowBackend_ = std::make_unique<InputWindowBackend>(this);

    bus_->addObjectVTable(inputWindowBackendPatch_,
                          inputWindowBackendInterface_, *inputWindowBackend_);

    bus_->requestName(inputWindowBackendName_,
                      fcitx::Flags<fcitx::dbus::RequestNameFlag>{
                          fcitx::dbus::RequestNameFlag::ReplaceExisting,
                          fcitx::dbus::RequestNameFlag::Queue});
    bus_->flush();
}

void InputWindowProxy::updateInputPanel(fcitx::InputContext *inputContext) {
    updateCursorPosition(inputContext);
    updatePreeditArea(inputContext);
    updateCandidateArea(inputContext);
}

void InputWindowProxy::updateCursorPosition(fcitx::InputContext *inputContext) {
    fcitx::Rect rect = inputContext->cursorRect();

    notifyCursorPosition(rect.left(), rect.top(), rect.width(), rect.height());
}

void InputWindowProxy::updatePreeditArea(fcitx::InputContext *inputContext) {
    auto &inputPanel = inputContext->inputPanel();
    auto preedit = instance_->outputFilter(inputContext, inputPanel.preedit());
    auto auxUp = instance_->outputFilter(inputContext, inputPanel.auxUp());
    auto preeditString = preedit.toString();
    auto auxUpString = auxUp.toString();
    size_t utf8Cursor = 0;
    if (!preeditString.empty() || !auxUpString.empty()) {
        auto text = auxUpString + preeditString;
        if (preedit.cursor() >= 0 &&
            static_cast<size_t>(preedit.cursor()) <= preeditString.size()) {
            auto cursor = preedit.cursor() + auxUpString.size();
            utf8Cursor = fcitx::utf8::lengthValidated(
                text.begin(), std::next(text.begin(), cursor));
            if (utf8Cursor == fcitx::utf8::INVALID_LENGTH) {
                utf8Cursor = 0;
            }
        }
    } else {
        preeditString = "";
        utf8Cursor = 0;
    }

    notifyPreeditCaret(utf8Cursor);
    notifyPreeditText(preeditString);
}

void InputWindowProxy::updateCandidateArea(fcitx::InputContext *inputContext) {
    std::vector<std::string> labels;
    std::vector<std::string> texts;
    std::vector<std::string> attrs;
    bool hasPrev = false, hasNext = false;
    int pos = -1;
    int localHighlightIdx = 0;

    auto &inputPanel = inputContext->inputPanel();
    auto auxDown = instance_->outputFilter(inputContext, inputPanel.auxDown());
    auto auxDownString = auxDown.toString();
    auto candidateList = inputPanel.candidateList();
    auto visible =
        !auxDownString.empty() || (candidateList && candidateList->size());
    if (visible) {
        if (!auxDownString.empty()) {
            labels.emplace_back("");
            texts.push_back(auxDownString);
            attrs.emplace_back("");
        }
        if (candidateList) {
            for (int i = 0, e = candidateList->size(); i < e; i++) {
                auto *commonCandidateList =
                    dynamic_cast<fcitx::CommonCandidateList *>(
                        candidateList.get());
                localHighlightIdx = commonCandidateList->globalCursorIndex();
                const auto &candidate = candidateList->candidate(i);
                if (candidate.isPlaceHolder()) {
                    continue;
                }
                fcitx::Text labelText = candidate.hasCustomLabel()
                                            ? candidate.customLabel()
                                            : candidateList->label(i);

                labelText = instance_->outputFilter(inputContext, labelText);
                labels.push_back(labelText.toString());
                auto candidateText =
                    instance_->outputFilter(inputContext, candidate.text());
                texts.push_back(candidateText.toString());
                attrs.emplace_back("");
            }
            if (auto *pageable = candidateList->toPageable()) {
                hasPrev = pageable->hasPrev();
                hasNext = pageable->hasNext();
            }
            pos = candidateList->cursorIndex();
            if (pos >= 0) {
                pos += (auxDownString.empty() ? 0 : 1);
            }
        }

        notifyCandidateArea(texts, hasPrev, hasNext, localHighlightIdx, pos);
    }
}

void InputWindowProxy::selectCandidate(int index) {
    auto *inputContext = instance_->mostRecentInputContext();
    if (inputContext == nullptr) {
        return;
    }

    if (auto candidateList = inputContext->inputPanel().candidateList()) {
        const auto *candidate =
            nthCandidateIgnorePlaceholder(*candidateList, index);
        if (candidate) {
            candidate->select(inputContext);
        }
    }
}

void InputWindowProxy::prevPage() {
    auto *inputContext = instance_->mostRecentInputContext();
    if (inputContext == nullptr) {
        return;
    }

    if (auto candidateList = inputContext->inputPanel().candidateList()) {
        if (auto *pageable = candidateList->toPageable()) {
            if (pageable->hasPrev()) {
                pageable->prev();
                inputContext->updateUserInterface(
                    fcitx::UserInterfaceComponent::InputPanel);
            }
        }
    }
}

void InputWindowProxy::nextPage() {
    auto *inputContext = instance_->mostRecentInputContext();
    if (inputContext == nullptr) {
        return;
    }

    if (auto candidateList = inputContext->inputPanel().candidateList()) {
        if (auto *pageable = candidateList->toPageable()) {
            if (pageable->hasNext()) {
                pageable->next();
                inputContext->updateUserInterface(
                    fcitx::UserInterfaceComponent::InputPanel);
            }
        }
    }
}

void InputWindowProxy::notifyCursorPosition(int x, int y, int width,
                                            int height) {
    auto msg = bus_->createMethodCall(uiServiceName_, uiServicePatch_,
                                      uiServiceInterface_, "UpdateCursorRect");
    msg << x << y << width << height;
    msg.send();
}

void InputWindowProxy::notifyPreeditText(const std::string &preeditString) {
    auto msg = bus_->createMethodCall(uiServiceName_, uiServicePatch_,
                                      uiServiceInterface_, "UpdatePreeditText");
    msg << preeditString;
    msg.send();
}

void InputWindowProxy::notifyPreeditCaret(int utf8Cursor) {
    auto msg =
        bus_->createMethodCall(uiServiceName_, uiServicePatch_,
                               uiServiceInterface_, "UpdatePreeditCaret");
    msg << utf8Cursor;
    msg.send();
}

void InputWindowProxy::notifyCandidateArea(
    const std::vector<std::string> &candidateTextList, bool hasPrev,
    bool hasNext, int pageIndex, int highlightIndex) {
    auto msg =
        bus_->createMethodCall(uiServiceName_, uiServicePatch_,
                               uiServiceInterface_, "UpdateCandidateArea");
    msg << candidateTextList << hasPrev << hasNext << pageIndex
        << highlightIndex;
    msg.send();
}
