/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "okinputmethodengine.h"

OKInputMethodEngine::OKInputMethodEngine(fcitx::Instance *instance)
    : instance_(instance) {
    pinyinEngine_ = std::make_unique<fcitx::PinyinEngine>(instance_);
}

OKInputMethodEngine::~OKInputMethodEngine() {}

void OKInputMethodEngine::activate(const fcitx::InputMethodEntry &entry,
                                   fcitx::InputContextEvent &event) {
    pinyinEngine_->activate(entry, event);
}

void OKInputMethodEngine::deactivate(const fcitx::InputMethodEntry &entry,
                                     fcitx::InputContextEvent &event) {
    pinyinEngine_->deactivate(entry, event);
}

void OKInputMethodEngine::keyEvent(const fcitx::InputMethodEntry &entry,
                                   fcitx::KeyEvent &keyEvent) {
    pinyinEngine_->keyEvent(entry, keyEvent);
}

void OKInputMethodEngine::reloadConfig() { pinyinEngine_->reloadConfig(); }

void OKInputMethodEngine::reset(const fcitx::InputMethodEntry &entry,
                                fcitx::InputContextEvent &event) {
    pinyinEngine_->reset(entry, event);
}

void OKInputMethodEngine::save() { pinyinEngine_->save(); }

std::string OKInputMethodEngine::subMode(const fcitx::InputMethodEntry &entry,
                                         fcitx::InputContext &inputContext) {
    return pinyinEngine_->subMode(entry, inputContext);
}

void OKInputMethodEngine::invokeActionImpl(const fcitx::InputMethodEntry &entry,
                                           fcitx::InvokeActionEvent &event) {
    pinyinEngine_->invokeActionImpl(entry, event);
}

const fcitx::Configuration *OKInputMethodEngine::getConfig() const {
    return pinyinEngine_->getConfig();
}

void OKInputMethodEngine::setConfig(const fcitx::RawConfig &config) {
    pinyinEngine_->setConfig(config);
}

void OKInputMethodEngine::setSubConfig(const std::string &path,
                                       const fcitx::RawConfig &config) {
    pinyinEngine_->setSubConfig(path, config);
}

const InputMethodStatus::StatusBarAction &OKInputMethodEngine::language() {
    languageAction_.actionType =
        static_cast<int>(InputMethodStatus::ActionType::Language);
    languageAction_.actionValue =
        static_cast<int>(InputMethodStatus::Language::Chinese);
    languageAction_.enable = false;
    languageAction_.shortcut = "";

    return languageAction_;
}

const InputMethodStatus::StatusBarAction &OKInputMethodEngine::chttransInfo() {
    chttransAction_.actionType =
        static_cast<int>(InputMethodStatus::ActionType::Chttrans);
    chttransAction_.actionValue =
        static_cast<int>(InputMethodStatus::Chttrans::Simplified);
    chttransAction_.enable = false;
    chttransAction_.shortcut = "Ctrl + F";
    chttrans_ = instance_->userInterfaceManager().lookupAction("chttrans");
    if (chttrans_ == nullptr) {
        return chttransAction_;
    }
    auto ic = instance_->mostRecentInputContext();
    if (ic == nullptr) {
        return chttransAction_;
    }

    if (chttrans_->icon(ic) == "fcitx-chttrans-active") {
        chttransAction_.actionValue =
            static_cast<int>(InputMethodStatus::Chttrans::Traditional);
    }
    chttransAction_.enable = true;
    return chttransAction_;
}

const InputMethodStatus::StatusBarAction &OKInputMethodEngine::charWidthInfo() {
    charWidthAction_.actionType =
        static_cast<int>(InputMethodStatus::ActionType::CharWidth);
    charWidthAction_.actionValue =
        static_cast<int>(InputMethodStatus::CharWidth::FullWidth);
    charWidthAction_.enable = false;
    charWidthAction_.shortcut = "";
    charWidth_ = instance_->userInterfaceManager().lookupAction("fullwidth");
    if (charWidth_ == nullptr) {
        return charWidthAction_;
    }
    auto ic = instance_->mostRecentInputContext();
    if (ic == nullptr) {
        return charWidthAction_;
    }

    if (charWidth_->icon(ic) == "fcitx-fullwidth-inactive") {
        charWidthAction_.actionValue =
            static_cast<int>(InputMethodStatus::CharWidth::HalfWidth);
    }
    chttransAction_.enable = true;
    return charWidthAction_;
}

const InputMethodStatus::StatusBarAction &
OKInputMethodEngine::punctuationInfo() {
    punctuationAction_.actionType =
        static_cast<int>(InputMethodStatus::ActionType::Punctuation);
    punctuationAction_.actionValue =
        static_cast<int>(InputMethodStatus::Punctuation::Chinese);
    punctuationAction_.enable = false;
    punctuationAction_.shortcut = "Ctrl + .";
    punctuation_ =
        instance_->userInterfaceManager().lookupAction("punctuation");
    if (punctuation_ == nullptr) {
        return punctuationAction_;
    }
    auto ic = instance_->mostRecentInputContext();
    if (ic == nullptr) {
        return punctuationAction_;
    }

    if (punctuation_->icon(ic) == "fcitx-punc-inactive") {
        punctuationAction_.actionValue =
            static_cast<int>(InputMethodStatus::CharWidth::HalfWidth);
    }
    punctuationAction_.enable = true;
    return punctuationAction_;
}

const InputMethodStatus::StatusBarAction &
OKInputMethodEngine::inputSchemeInfo() {
    inputSchemeAction_.actionType =
        static_cast<int>(InputMethodStatus::ActionType::InputScheme);
    inputSchemeAction_.actionValue =
        static_cast<int>(InputMethodStatus::PinyinScheme::Quanpin);
    inputSchemeAction_.enable = false;
    inputSchemeAction_.shortcut = "";

    return inputSchemeAction_;
}

bool OKInputMethodEngine::switchStatusActionBar(int actionType) const {
    InputMethodStatus::ActionType type =
        static_cast<InputMethodStatus::ActionType>(actionType);
    auto ic = instance_->mostRecentInputContext();
    if (ic == nullptr) {
        return false;
    }

    if ((type == InputMethodStatus::ActionType::CharWidth) &&
        (instance_->userInterfaceManager().lookupAction("fullwidth") !=
         nullptr)) {

        charWidth_->activate(ic);
    } else if ((type == InputMethodStatus::ActionType::Punctuation) &&
               (instance_->userInterfaceManager().lookupAction("punctuation") !=
                nullptr)) {

        punctuation_->activate(ic);
    } else if ((type == InputMethodStatus::ActionType::Chttrans) &&
               (instance_->userInterfaceManager().lookupAction("chttrans") !=
                nullptr)) {

        chttrans_->activate(ic);
    } else {
        return false;
    }

    return true;
}