/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "statusbarproxy.h"

static const char uiServiceName_[] = "org.ukui.okinputmethod";
static const char uiServicePatch_[] = "/org/ukui/okinputmethod";
static const char uiServiceInterface_[] = "org.ukui.okinputmethod";

static const char statusBarBackendName_[] = "org.ukui.inputmethodbackend";
static const char statusBarBackendPath_[] = "/statusbarproxy";
static const char statusBarBackendInterface_[] =
    "org.ukui.inputmethod.statusbarproxy";

class StatusBarBackend : public fcitx::dbus::ObjectVTable<StatusBarBackend> {
public:
    explicit StatusBarBackend(StatusBarProxy *parent) : parent_(parent) {}

    ~StatusBarBackend() = default;

    std::vector<fcitx::dbus::DBusStruct<int, int, bool, std::string>>
    getStatusBarActions() {
        std::vector<fcitx::dbus::DBusStruct<int, int, bool, std::string>>
            actions_;
        for (const InputMethodStatus::StatusBarAction &action :
             parent_->currentStatusBarActions()) {
            actions_.emplace_back(action.actionType, action.actionValue,
                                  action.enable, action.shortcut);
        }

        return actions_;
    }

    bool setStatusBarAction(int index) const {
        return parent_->setStatusBarAction(index);
    }

private:
    FCITX_OBJECT_VTABLE_METHOD(getStatusBarActions, "GetStatusBarActions", "",
                               "a(iibs)");

    FCITX_OBJECT_VTABLE_METHOD(setStatusBarAction, "SetStatusBarAction", "i",
                               "b");

private:
    StatusBarProxy *parent_;
};

StatusBarProxy::StatusBarProxy(EngineManager *engineManager,
                               fcitx::dbus::Bus *bus)
    : engineManager_(engineManager), bus_(bus) {
    initStatusBarBackend();
}

StatusBarProxy::~StatusBarProxy() = default;

void StatusBarProxy::imeActivated() {
    actions_.clear();
    currentIMEActivated = true;
    updateStatusBarActions();
    notifyStatusBarArea();
    notifyIMEActivated();
}

void StatusBarProxy::imeDeactivated() {
    notifyIMEDeactivated();
    actions_.clear();
    currentIMEActivated = false;
}

void StatusBarProxy::updateStatusBar() {
    actions_.clear();
    updateStatusBarActions();
    notifyStatusBarArea();
}

void StatusBarProxy::initStatusBarBackend() {
    statusBarBackend_ = std::make_unique<StatusBarBackend>(this);

    bus_->addObjectVTable(statusBarBackendPath_, statusBarBackendInterface_,
                          *statusBarBackend_);

    bus_->requestName(statusBarBackendName_,
                      fcitx::Flags<fcitx::dbus::RequestNameFlag>{
                          fcitx::dbus::RequestNameFlag::ReplaceExisting,
                          fcitx::dbus::RequestNameFlag::Queue});
    bus_->flush();
}

void StatusBarProxy::notifyIMEActivated() {
    auto msg = bus_->createMethodCall(uiServiceName_, uiServicePatch_,
                                      uiServiceInterface_,
                                      "InputMethodEngineActivated");
    msg.send();
}

void StatusBarProxy::notifyIMEDeactivated() {
    auto msg = bus_->createMethodCall(uiServiceName_, uiServicePatch_,
                                      uiServiceInterface_,
                                      "InputMethodEngineDeactivated");
    msg.send();
}

void StatusBarProxy::notifyStatusBarArea() {
    std::vector<fcitx::dbus::DBusStruct<int, int, bool, std::string>>
        actionsTmp_;
    for (const InputMethodStatus::StatusBarAction &action : actions_) {
        actionsTmp_.emplace_back(action.actionType, action.actionValue,
                                 action.enable, action.shortcut);
    }

    auto msg =
        bus_->createMethodCall(uiServiceName_, uiServicePatch_,
                               uiServiceInterface_, "UpdateStatusBarArea");
    msg << actionsTmp_;
    msg.send();
}

std::vector<InputMethodStatus::StatusBarAction> &
StatusBarProxy::currentStatusBarActions() {
    if (!currentIMEActivated) {
        actions_.clear();
        return actions_;
    }

    return actions_;
}

void StatusBarProxy::updateStatusBarActions() {
    actions_.push_back(engineManager_->currentInputMethodEngine().language());
    actions_.push_back(
        engineManager_->currentInputMethodEngine().inputSchemeInfo());
    actions_.push_back(
        engineManager_->currentInputMethodEngine().chttransInfo());
    actions_.push_back(
        engineManager_->currentInputMethodEngine().charWidthInfo());
    actions_.push_back(
        engineManager_->currentInputMethodEngine().punctuationInfo());
}

bool StatusBarProxy::setStatusBarAction(int index) const {
    return engineManager_->currentInputMethodEngine().switchStatusActionBar(
        index);
}
