/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "inputwindowmanager.h"

#include <QQmlEngine>

#include "inputwindowmodel.h"

InputWindowManager::InputWindowManager(QObject *parent)
    : QObject(parent), m_inputWindowModel(new InputWindowModel(this)),
      m_inputWindowView(new InputWindowView(*m_inputWindowModel, *this)) {
    // 将 InputWindowModel 注册为 “不可创建类型”，但是 QML
    // 可以访问类里面的枚举值
    qmlRegisterUncreatableType<InputWindowModel>(
        "InputWindowModelType", 1, 0, "InputWindowModel",
        "Not be instantiated in QML, we should only pass the enum types to QML "
        "!");
}

InputWindowManager::~InputWindowManager() {}

void InputWindowManager::updateCursorRect(const QRect &cursorRect) {
    m_inputWindowModel->setCursorPos(cursorRect);
}

void InputWindowManager::updatePreeditText(const QString &preeditText) {
    if (preeditText.isEmpty()) {
        hideInputWindow();
    }
    m_inputWindowModel->setPreeditText(preeditText);
}

void InputWindowManager::updatePreeditCaret(int caretIndex) {
    m_inputWindowModel->setPreeditCaret(caretIndex);
}

void InputWindowManager::updateCandidateArea(
    const QStringList &candidateTextList, bool hasPrev, bool hasNext,
    int pageIndex, int localHighlightIdx) {
    if (candidateTextList.size() > 0) {
        m_inputWindowModel->resetCandidates(candidateTextList);
        m_inputWindowModel->setHasPrev(hasPrev);
        m_inputWindowModel->setHasNext(hasNext);
        m_inputWindowModel->setPageIndex(pageIndex);
        m_inputWindowModel->setHighlightIdx(localHighlightIdx);
        showInputWindow();
    } else {
        hideInputWindow();
    }
}

void InputWindowManager::showInputWindow() { m_inputWindowView->show(); }

void InputWindowManager::hideInputWindow() { m_inputWindowView->hide(); }

void InputWindowManager::setSelectCandidateCallback(
    SelectCandidateCallback callback) {
    m_selectCandidateCallback = std::move(callback);
}

void InputWindowManager::setPrevPageCallback(PrevPageCallback callback) {
    m_prevPageCallback = std::move(callback);
}

void InputWindowManager::setNextPageCallback(NextPageCallback callback) {
    m_nextPageCallback = std::move(callback);
}

void InputWindowManager::selectCandidate(int localIdx) {
    if (m_selectCandidateCallback == nullptr) {
        return;
    }

    m_selectCandidateCallback(localIdx);
}

void InputWindowManager::prevPage() {
    if (m_prevPageCallback == nullptr) {
        return;
    }

    m_prevPageCallback();
}

void InputWindowManager::nextPage() {
    if (m_nextPageCallback == nullptr) {
        return;
    }

    m_nextPageCallback();
}
