/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "inputwindowmodel.h"
#include <QDebug>

Candidate::Candidate(int index, const QString &text, QObject *parent)
    : m_index(index), m_text(text), QObject(parent) {}

int Candidate::index() const { return m_index; }

QString Candidate::text() const { return m_text; }

bool Candidate::highlighted() const { return m_highlighted; }

void Candidate::setHighlighted(bool highlighted) {
    if (m_highlighted == highlighted) {
        return;
    }
    m_highlighted = highlighted;
    emit highlightedChanged();
}

CandidateModel::CandidateModel(QObject *parent) : QAbstractListModel(parent) {}

void CandidateModel::resetCandidates(const QStringList &candList) {
    clearModelData();
    addModelData(candList);
}

void CandidateModel::setHighlightIdx(const int highlightIdx) {
    for (int index = 0; index < m_candidates.size(); ++index) {
        m_candidates.at(index)->setHighlighted(index == highlightIdx);
    }
}

int CandidateModel::rowCount(const QModelIndex &parent) const {
    Q_UNUSED(parent);
    return m_candidates.count();
}

QVariant CandidateModel::data(const QModelIndex &index, int role) const {
    if (index.row() < 0 || index.row() >= m_candidates.count())
        return QVariant();

    if (role == CandidateRole) {
        Candidate *candidate = m_candidates[index.row()];
        qDebug() << __func__ << candidate->index() << candidate->text()
                 << candidate->highlighted();
        return QVariant::fromValue(candidate);
    }

    return QVariant();
}

QHash<int, QByteArray> CandidateModel::roleNames() const {
    QHash<int, QByteArray> roles;
    roles[CandidateRole] = "candidate";
    return roles;
}

void CandidateModel::clearModelData() {
    beginRemoveRows(QModelIndex(), 0, m_candidates.count() - 1);
    qDeleteAll(m_candidates);
    m_candidates.clear();
    endRemoveRows();
}

void CandidateModel::addModelData(const QStringList &candList) {
    beginInsertRows(QModelIndex(), 0, candList.count() - 1);
    for (int index = 0; index < candList.size(); ++index) {
        m_candidates.append(new Candidate(index + 1, candList.at(index)));
    }
    endInsertRows();
}

InputWindowModel::InputWindowModel(QObject *parent) : QObject(parent) {}

QAbstractListModel *InputWindowModel::candidateModel() {
    return static_cast<QAbstractListModel *>(&m_candidateModel);
}

void InputWindowModel::setCursorPos(const QRect &pos) {
    if (m_cursorPos == pos) {
        return;
    }
    m_cursorPos = pos;
    emit cursorPosChanged();
}

void InputWindowModel::setLayout(const InputWindowModel::Layout layout) {
    if (m_layout == layout) {
        return;
    }
    m_layout = layout;
}

void InputWindowModel::setPreeditText(const QString &text) {
    if (m_preeditText == text) {
        return;
    }
    m_preeditText = text;
    emit preeditTextChanged();
}

void InputWindowModel::setPreeditCaret(const int caretIdx) {
    if (m_preeditCaret == caretIdx) {
        return;
    }
    m_preeditCaret = caretIdx;
    emit preeditCaretChanged();
}

void InputWindowModel::setPageIndex(const int pageIndex) {
    if (m_pageIndex == pageIndex) {
        return;
    }
    m_pageIndex = pageIndex;
    emit pageIndexChanged();
}

void InputWindowModel::resetCandidates(const QStringList &candList) {
    m_candidateModel.resetCandidates(candList);
}

void InputWindowModel::setHighlightIdx(const int highlightIdx) {
    m_candidateModel.setHighlightIdx(highlightIdx);
}

void InputWindowModel::setHasPrev(bool hasPrev) {
    if (m_hasPrev == hasPrev) {
        return;
    }
    m_hasPrev = hasPrev;
    emit hasPrevChanged();
}

void InputWindowModel::setHasNext(bool hasNext) {
    if (m_hasNext == hasNext) {
        return;
    }
    m_hasNext = hasNext;
    emit hasNextChanged();
}
