/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef INPUTWINDOWMODEL_H
#define INPUTWINDOWMODEL_H

#include <QAbstractListModel>
#include <QObject>
#include <QRect>
#include <QStringList>

class Candidate : public QObject {
    Q_OBJECT
    Q_PROPERTY(int index READ index CONSTANT)
    Q_PROPERTY(QString text READ text CONSTANT)
    Q_PROPERTY(bool highlighted READ highlighted WRITE setHighlighted NOTIFY
                   highlightedChanged)

public:
    Candidate(int index, const QString &text, QObject *parent = nullptr);
    virtual ~Candidate() = default;

    int index() const;
    QString text() const;
    bool highlighted() const;
    void setHighlighted(bool highlighted);

signals:
    void highlightedChanged();

private:
    int m_index;
    QString m_text;
    bool m_highlighted = false;
};

class CandidateModel : public QAbstractListModel {
    Q_OBJECT
public:
    enum CandidateRoles { CandidateRole = Qt::UserRole + 1 };

    CandidateModel(QObject *parent = 0);
    virtual ~CandidateModel() = default;
    void resetCandidates(const QStringList &candList);
    void setHighlightIdx(const int highlightIdx);
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;

protected:
    QHash<int, QByteArray> roleNames() const override;

private:
    void clearModelData();
    void addModelData(const QStringList &candList);

private:
    QList<Candidate *> m_candidates;
};

class InputWindowModel : public QObject {
    Q_OBJECT
    Q_PROPERTY(QRect cursorPos READ cursorPos NOTIFY cursorPosChanged)
    Q_PROPERTY(Layout layout READ layout WRITE setLayout)
    Q_PROPERTY(QString preeditText READ preeditText NOTIFY preeditTextChanged)
    Q_PROPERTY(int preeditCaret READ preeditCaret NOTIFY preeditCaretChanged)
    Q_PROPERTY(int pageIndex READ pageIndex NOTIFY pageIndexChanged)
    Q_PROPERTY(QAbstractListModel *candidateModel READ candidateModel CONSTANT)
    Q_PROPERTY(bool hasPrev READ hasPrev NOTIFY hasPrevChanged)
    Q_PROPERTY(bool hasNext READ hasNext NOTIFY hasNextChanged)

public:
    explicit InputWindowModel(QObject *parent = nullptr);
    virtual ~InputWindowModel() = default;

    enum class Layout { HorizontalLayout, VerticalLayout };
    Q_ENUM(Layout)

    QRect cursorPos() const { return m_cursorPos; }
    Layout layout() const { return m_layout; }
    QString preeditText() const { return m_preeditText; }
    int preeditCaret() const { return m_preeditCaret; }
    int pageIndex() const { return m_pageIndex; }
    QAbstractListModel *candidateModel();
    bool hasPrev() const { return m_hasPrev; }
    bool hasNext() const { return m_hasNext; }

    void setCursorPos(const QRect &pos);
    void setLayout(const Layout layout);
    void setPreeditText(const QString &text);
    void setPreeditCaret(const int caretIdx);
    void setPageIndex(const int pageIndex);
    void resetCandidates(const QStringList &candList);
    void setHighlightIdx(const int highlightIdx);
    void setHasPrev(bool hasPrev);
    void setHasNext(bool hasNext);

    void selectCandidate(int globalIdx);
    void prevPage();
    void nextPage();

signals:
    void preeditTextChanged();
    void preeditCaretChanged();
    void pageIndexChanged();
    void cursorPosChanged();
    void hasPrevChanged();
    void hasNextChanged();

private:
    QRect m_cursorPos;
    Layout m_layout;
    QString m_preeditText;
    int m_preeditCaret;
    int m_pageIndex;
    CandidateModel m_candidateModel;
    bool m_hasPrev;
    bool m_hasNext;
};

#endif // INPUTWINDOWMODEL_H
