/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "inputwindowview.h"
#include "windowmanager/windowmanager.h"

#include <QQmlContext>

InputWindowView::InputWindowView(InputWindowModel &model, QObject &manager)
    : m_model(model), m_manager(manager),
      m_geometryManager(new InputWindowGeometryManager()) {
    initView();
    connectSignals();
}

void InputWindowView::show() {
    updateInputWindowPos();
    m_view.show();
}

void InputWindowView::hide() { m_view.hide(); }

void InputWindowView::initView() {
    m_view.rootContext()->setContextProperty("inputWindowModel", &m_model);
    m_view.rootContext()->setContextProperty("inputWindowManager", &m_manager);

    m_view.setSource(QUrl("qrc:/qml/InputWindow.qml"));
    m_view.setColor(QColor(Qt::transparent));
    m_view.setFlags(Qt::Window | Qt::FramelessWindowHint |
                    Qt::WindowStaysOnTopHint | Qt::BypassWindowManagerHint);
}

void InputWindowView::connectSignals() {
    QObject::connect(&m_view, &QQuickView::widthChanged,
                     [this]() { updateInputWindowPos(); });
    QObject::connect(&m_view, &QQuickView::heightChanged,
                     [this]() { updateInputWindowPos(); });

    QObject::connect(&m_model, &InputWindowModel::cursorPosChanged, [this]() {
        updateCursorPosition();
        updateInputWindowPos();
    });
}

QPoint InputWindowView::getPosition() {
    int x = m_model.cursorPos().x() / m_view.devicePixelRatio();
    int y = m_model.cursorPos().y() / m_view.devicePixelRatio();
    return m_geometryManager->getValidPostionByInitialGeometry(
        QRect(QPoint(x, y), m_view.size()));
}

void InputWindowView::updateCursorPosition() {
    qreal ratio = m_view.devicePixelRatio();
    QRect scaledCursorPosition = {
        static_cast<int>(m_model.cursorPos().x() / ratio),
        static_cast<int>(m_model.cursorPos().y() / ratio),
        static_cast<int>(m_model.cursorPos().width() / ratio),
        static_cast<int>(m_model.cursorPos().height() / ratio)};
    m_geometryManager->setCursorPosition(scaledCursorPosition);
}

void InputWindowView::updateInputWindowPos() {
    kdk::WindowManager::setGeometry(&m_view,
                                    QRect(getPosition().x(), getPosition().y(),
                                          m_view.width(), m_view.height()));
}
