/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "dbuschannel.h"

#include <QCoreApplication>
#include <QDBusConnection>
#include <QDBusError>
#include <QDBusMetaType>
#include <QDBusPendingCall>
#include <QDBusPendingReply>
#include <QDebug>

#include "inputwindowmanager.h"
#include "statusbarmanager.h"

using namespace InputMethodStatus;

DBusChannel::DBusChannel(InputWindowManager &inputWindowManager,
                         StatusBarManager &statusBarManager, QObject *parent)
    : m_inputWindowManager(inputWindowManager),
      m_statusBarManager(statusBarManager), QObject(parent) {
    setCallback();
    startDBusService();
    initInputMethodBackendInterface();
    initInputMethodBackendServiceWatcher();
}

void DBusChannel::InputMethodEngineActivated() {
    qDebug() << "ok input method started ! ! !";
    requestStatusBarActions();
}

void DBusChannel::InputMethodEngineDeactivated() {
    m_inputWindowManager.hideInputWindow();
    m_statusBarManager.hideStatusBar();
}

void DBusChannel::UpdateCursorRect(int x, int y, int width, int height) {
    m_inputWindowManager.updateCursorRect(QRect(x, y, width, height));
}

void DBusChannel::UpdatePreeditText(const QString &preeditText) {
    m_inputWindowManager.updatePreeditText(preeditText);
}

void DBusChannel::UpdatePreeditCaret(int caretIndex) {
    m_inputWindowManager.updatePreeditCaret(caretIndex);
}

void DBusChannel::UpdateCandidateArea(const QStringList &candidateTextList,
                                      bool hasPrev, bool hasNext, int pageIndex,
                                      int localHighlightIdx) {
    m_inputWindowManager.updateCandidateArea(
        candidateTextList, hasPrev, hasNext, pageIndex, localHighlightIdx);
}

void DBusChannel::UpdateStatusBarArea(QList<StatusBarAction> actionList) {
    m_statusBarManager.updateStatusBarArea(actionList);
}

bool DBusChannel::startDBusService() {
    QDBusConnection connection = QDBusConnection::sessionBus();

    if (!connection.registerService(m_serviceName)) {
        qWarning() << "Warning: " << connection.lastError().message();
        return false;
    }

    int ret = connection.registerObject(m_servicePath, m_serviceInterface, this,
                                        QDBusConnection::ExportAllSlots);
    if (!ret) {
        qWarning() << "Warning: " << connection.lastError().message();
        return false;
    }

    qDBusRegisterMetaType<StatusBarAction>();
    qDBusRegisterMetaType<QList<StatusBarAction>>();
    return true;
}

void DBusChannel::setCallback() {
    m_inputWindowManager.setSelectCandidateCallback(
        [this](int currentIndex) { this->selectCandidate(currentIndex); });
    m_inputWindowManager.setPrevPageCallback([this]() { this->prevPage(); });
    m_inputWindowManager.setNextPageCallback([this]() { this->nextPage(); });
    m_statusBarManager.setSwitchStatusBarActionCallback(
        [this](int actionType) { this->switchStatusBarAction(actionType); });
    m_statusBarManager.setRequestStatusBarActionsCallback(
        [this]() { this->requestStatusBarActions(); });
}

void DBusChannel::initInputMethodBackendInterface() {
    m_inputWindowBackendInterface.reset(new QDBusInterface(
        m_inputMethodBackendServiceName, m_inputWindowPath,
        m_inputWindowInterface, QDBusConnection::sessionBus(), this));
    m_statusBarBackendInterface.reset(new QDBusInterface(
        m_inputMethodBackendServiceName, m_statusBarPath, m_statusBarInterface,
        QDBusConnection::sessionBus(), this));
}

void DBusChannel::initInputMethodBackendServiceWatcher() {
    m_inputMethodBackendServiceWatcher.reset(new QDBusServiceWatcher(this));
    m_inputMethodBackendServiceWatcher->setConnection(
        QDBusConnection::sessionBus());
    m_inputMethodBackendServiceWatcher->addWatchedService(
        m_inputMethodBackendServiceName);
    m_inputMethodBackendServiceWatcher->setWatchMode(
        QDBusServiceWatcher::WatchForRegistration |
        QDBusServiceWatcher::WatchForUnregistration);
    connect(m_inputMethodBackendServiceWatcher.get(),
            &QDBusServiceWatcher::serviceRegistered, this,
            &DBusChannel::inputMethodBackendServiceRegistered);
    connect(m_inputMethodBackendServiceWatcher.get(),
            &QDBusServiceWatcher::serviceUnregistered, this,
            &DBusChannel::inputMethodBackendServiceUnregistered);
}

void DBusChannel::inputMethodBackendServiceRegistered(
    const QString &serviceName) {
    initInputMethodBackendInterface();
}

void DBusChannel::inputMethodBackendServiceUnregistered(
    const QString &serviceName) {
    m_inputWindowBackendInterface.reset();
    m_statusBarBackendInterface.reset();
}

void DBusChannel::closeApp() { qApp->quit(); }

void DBusChannel::selectCandidate(int localIdx) {
    if (m_inputWindowBackendInterface == nullptr) {
        return;
    }
    m_inputWindowBackendInterface->asyncCall(QLatin1String("SelectCandidate"),
                                             localIdx);
}

void DBusChannel::prevPage() {
    if (m_inputWindowBackendInterface == nullptr) {
        return;
    }
    m_inputWindowBackendInterface->asyncCall(QLatin1String("PrevPage"));
}

void DBusChannel::nextPage() {
    if (m_inputWindowBackendInterface == nullptr) {
        return;
    }
    m_inputWindowBackendInterface->asyncCall(QLatin1String("NextPage"));
}

void DBusChannel::switchStatusBarAction(int index) {
    if (m_statusBarBackendInterface == nullptr) {
        return;
    }
    m_statusBarBackendInterface->asyncCall(QLatin1String("SetStatusBarAction"),
                                           index);
}

void DBusChannel::requestStatusBarActions() {
    if (m_statusBarBackendInterface == nullptr) {
        return;
    }
    QDBusPendingCall pcall = m_statusBarBackendInterface->asyncCall(
        QLatin1String("GetStatusBarActions"));
    QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(pcall, this);
    connect(watcher, &QDBusPendingCallWatcher::finished, this,
            [this](QDBusPendingCallWatcher *call) {
                QDBusPendingReply<QList<StatusBarAction>> reply = *call;
                if (reply.isError()) {
                    qWarning() << "The statusbar data request is failed !";
                } else {
                    QList<StatusBarAction> actions = reply.value();
                    m_statusBarManager.updateStatusBarArea(actions);
                    if (actions.size() > 0) {
                        m_statusBarManager.showStatusBar();
                    }
                }
                call->deleteLater();
            });
}
