/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "statusbarmanager.h"

#include <QGuiApplication>
#include <QScreen>

#include "statusbarview.h"
#include "tooltip.h"

StatusBarManager::StatusBarManager(QObject *parent) : QObject(parent) {
    qmlRegisterType<ToolTip>("ToolTip", 2, 0, "ToolTipArea");
    QPoint initialPos = getPersistentStatusBarPos();
    m_statusBarView =
        new StatusBarView(m_statusBarModel, *this, initialPos, this);
    QTimer::singleShot(0, [this]() { m_requestStatusBarActionsCallback(); });
}

void StatusBarManager::updateStatusBarArea(
    QList<InputMethodStatus::StatusBarAction> &actionList) {
    if (actionList.size() == 0) {
        hideStatusBar();
        return;
    }
    sortStatusBarActions(actionList);
    m_statusBarModel.updateStatusBarButton(actionList);
}

void StatusBarManager::showStatusBar() { m_statusBarView->show(); }

void StatusBarManager::hideStatusBar() {
    QPoint pos = m_statusBarView->statusBarPosition();
    m_statusBarSettings.saveStatusBarPos(pos.x(), pos.y());
    m_statusBarView->hide();
}

void StatusBarManager::setSwitchStatusBarActionCallback(
    SwitchStatusBarActionCallback callback) {
    m_switchStatusBarActionCallback = std::move(callback);
}

void StatusBarManager::setRequestStatusBarActionsCallback(
    RequestStatusBarActionCallback callback) {
    m_requestStatusBarActionsCallback = std::move(callback);
}

void StatusBarManager::switchStatusBarAction(int index) {
    if (m_switchStatusBarActionCallback == nullptr) {
        return;
    }
    if (index >= actionsSequence.size() || index < 0) {
        return;
    }
    m_switchStatusBarActionCallback(actionsSequence[index]);
}

QPoint StatusBarManager::getPersistentStatusBarPos() {
    StatusBarSettings::StatusBarPositionData posData =
        m_statusBarSettings.getStatusBarPos();
    QPoint initialPos(posData.globalX, posData.globalY);

    // 位置信息存储着状态栏 x, y
    // 坐标的左边缘和上边缘占比，占比信息更能还原状态栏的相对位置
    return recalculateStatusBarPos(initialPos, posData.regionalLeftRatio,
                                   posData.regionalTopRatio);
}

QPoint StatusBarManager::recalculateStatusBarPos(const QPoint &globalPos,
                                                 float leftRatio,
                                                 float topRatio) {
    QPoint statusBarPos;
    QScreen *currentScreen = qApp->screenAt(globalPos);
    QRect screenGeo;
    // 本地持久化的状态栏位置不在本次系统启动屏幕范围内
    if (currentScreen == nullptr) {
        screenGeo = qApp->primaryScreen()->availableGeometry();
    } else {
        screenGeo = currentScreen->availableGeometry();
    }
    int globalX = screenGeo.x() + screenGeo.width() * leftRatio;
    int globalY = screenGeo.y() + screenGeo.height() * topRatio;
    statusBarPos.setX(globalX);
    statusBarPos.setY(globalY);
    return statusBarPos;
}

void StatusBarManager::sortStatusBarActions(
    QList<InputMethodStatus::StatusBarAction> &actionList) {
    using namespace InputMethodStatus;
    for (int i = 0, size = actionList.size(); i < size; i++) {
        for (int j = i + 1; j < size; j++) {
            if (actionList[j].actionType == actionsSequence[i]) {
                actionList.swap(i, j);
            }
        }
    }
}