#include "statusbarmodel.h"

namespace {
const QMap<InputMethodStatus::Language, QString> languageIconMap = {
    {InputMethodStatus::Language::Chinese, "ukui-chinese-symbolic"},
    {InputMethodStatus::Language::English, "ukui-english-symbolic"}};
const QMap<InputMethodStatus::CharWidth, QString> charWidthIconMap = {
    {InputMethodStatus::CharWidth::FullWidth, "ukui-full-width-symbolic"},
    {InputMethodStatus::CharWidth::HalfWidth, "ukui-half-width-symbolic"}};
const QMap<InputMethodStatus::Punctuation, QString> punctuationIconMap = {
    {InputMethodStatus::Punctuation::Chinese,
     "ukui-chinese-punctuation-symbolic"},
    {InputMethodStatus::Punctuation::English,
     "ukui-english-punctuation-symbolic"}};
const QMap<InputMethodStatus::Chttrans, QString> chttransIconMap = {
    {InputMethodStatus::Chttrans::Simplified, "ukui-simplified-symbolic"},
    {InputMethodStatus::Chttrans::Traditional, "ukui-traditional-symbolic"}};
const QMap<InputMethodStatus::PinyinScheme, QString> pinyinSchemeIconMap = {
    {InputMethodStatus::PinyinScheme::Quanpin, "ukui-pin-quan-symbolic"},
    {InputMethodStatus::PinyinScheme::Shuangpin, "ukui-pin-shuang-symbolic"}};
} // namespace

StatusBarModel::StatusBarModel(QObject *parent) : QAbstractListModel(parent) {}

QHash<int, QByteArray> StatusBarModel::roleNames() const {
    QHash<int, QByteArray> roleNames{
        {StatusBarModel::ModelDataRole, "modelData"}};
    return roleNames;
}

int StatusBarModel::rowCount(const QModelIndex &parent) const {
    Q_UNUSED(parent);
    return m_statusBarButtonList.count();
}

QVariant StatusBarModel::data(const QModelIndex &index, int role) const {
    if (index.row() < 0 || index.row() >= m_statusBarButtonList.count() ||
        ModelDataRole != role) {
        return QVariant();
    }
    return QVariant::fromValue(m_statusBarButtonList.at(index.row()).get());
}

void StatusBarModel::updateStatusBarButton(
    const QList<InputMethodStatus::StatusBarAction> &actionList) {
    clearModelData();
    addModelData(actionList);
}

void StatusBarModel::clearModelData() {
    beginRemoveRows(QModelIndex(), 0, m_statusBarButtonList.count() - 1);
    m_statusBarButtonList.clear();
    endRemoveRows();
}

void StatusBarModel::addModelData(
    const QList<InputMethodStatus::StatusBarAction> &actionList) {
    beginInsertRows(QModelIndex(), 0, actionList.count() - 1);
    for (const InputMethodStatus::StatusBarAction &action : actionList) {
        m_statusBarButtonList.append(createStatusBarButton(action));
    }
    endInsertRows();
}

std::shared_ptr<StatusBarItem> StatusBarModel::createStatusBarButton(
    InputMethodStatus::StatusBarAction action) {
    std::shared_ptr<StatusBarItem> statusBarItem(new StatusBarItem);
    statusBarItem->setIsClickable(action.enable);
    setStatusBarButtonProperty(statusBarItem, action);
    return statusBarItem;
}

void StatusBarModel::setStatusBarButtonProperty(
    std::shared_ptr<StatusBarItem> item,
    InputMethodStatus::StatusBarAction statusBarAction) {
    using namespace InputMethodStatus;

    switch ((ActionType)statusBarAction.actionType) {
    case ActionType::Language:
        setLanguageToolTip(item, QString(statusBarAction.shortcut.data()));
        setLanguageIcon(item, ((Language)statusBarAction.actionValue));
        break;
    case ActionType::CharWidth:
        setCharWidthToolTip(item, QString(statusBarAction.shortcut.data()));
        setCharWidthIcon(item, ((CharWidth)statusBarAction.actionValue));
        break;
    case ActionType::Punctuation:
        setPunctuationToolTip(item, QString(statusBarAction.shortcut.data()));
        setPunctuationIcon(item, ((Punctuation)statusBarAction.actionValue));
        break;
    case ActionType::Chttrans:
        setChttransToolTip(item, QString(statusBarAction.shortcut.data()));
        setChttransIcon(item, ((Chttrans)statusBarAction.actionValue));
        break;
    case ActionType::InputScheme:
        setInputSchemeToolTip(item, (QString(statusBarAction.shortcut.data())));
        setInputSchemeIcon(item, ((PinyinScheme)statusBarAction.actionValue));
        break;
    default:
        break;
    }
}

void StatusBarModel::setLanguageToolTip(std::shared_ptr<StatusBarItem> item,
                                        const QString &toolTip) {
    toolTip.isEmpty() ? item->setToolTip("中/英文")
                      : item->setToolTip("中/英文（" + toolTip + "）");
}

void StatusBarModel::setCharWidthToolTip(std::shared_ptr<StatusBarItem> item,
                                         const QString &toolTip) {
    toolTip.isEmpty() ? item->setToolTip("全/半角")
                      : item->setToolTip("全/半角（" + toolTip + "）");
}

void StatusBarModel::setPunctuationToolTip(std::shared_ptr<StatusBarItem> item,
                                           const QString &toolTip) {
    toolTip.isEmpty() ? item->setToolTip("中/英文符号")
                      : item->setToolTip("中/英文符号（" + toolTip + "）");
}

void StatusBarModel::setChttransToolTip(std::shared_ptr<StatusBarItem> item,
                                        const QString &toolTip) {
    toolTip.isEmpty() ? item->setToolTip("简/繁体")
                      : item->setToolTip("简/繁体（" + toolTip + "）");
}

void StatusBarModel::setInputSchemeToolTip(std::shared_ptr<StatusBarItem> item,
                                           const QString &toolTip) {
    toolTip.isEmpty() ? item->setToolTip("全/双拼")
                      : item->setToolTip("全/双拼（" + toolTip + "）");
}

void StatusBarModel::setLanguageIcon(std::shared_ptr<StatusBarItem> item,
                                     InputMethodStatus::Language language) {
    if (languageIconMap.keys().contains(language)) {
        item->setIconName(languageIconMap.value(language));
    }
}

void StatusBarModel::setCharWidthIcon(std::shared_ptr<StatusBarItem> item,
                                      InputMethodStatus::CharWidth charWidth) {
    if (charWidthIconMap.keys().contains(charWidth)) {
        item->setIconName(charWidthIconMap.value(charWidth));
    }
}

void StatusBarModel::setPunctuationIcon(
    std::shared_ptr<StatusBarItem> item,
    InputMethodStatus::Punctuation punctuation) {
    if (punctuationIconMap.keys().contains(punctuation)) {
        item->setIconName(punctuationIconMap.value(punctuation));
    }
}

void StatusBarModel::setChttransIcon(std::shared_ptr<StatusBarItem> item,
                                     InputMethodStatus::Chttrans chttrans) {
    if (chttransIconMap.keys().contains(chttrans)) {
        item->setIconName(chttransIconMap.value(chttrans));
    }
}

void StatusBarModel::setInputSchemeIcon(
    std::shared_ptr<StatusBarItem> item,
    InputMethodStatus::PinyinScheme inputScheme) {
    if (pinyinSchemeIconMap.keys().contains(inputScheme)) {
        item->setIconName(pinyinSchemeIconMap.value(inputScheme));
    }
}
