/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "statusbarview.h"
#include "windowmanager/windowmanager.h"

#include <QGuiApplication>
#include <QQmlContext>
#include <QScreen>

StatusBarView::StatusBarView(QObject &model, QObject &manager,
                             QPoint statusBarPos, QObject *parent)
    : QObject(parent), m_model(model), m_manager(manager) {
    initView(statusBarPos);
    initConnections();
}

QPoint StatusBarView::statusBarPosition() const {
    return QPoint(m_view.x(), m_view.y());
}

void StatusBarView::show() {
    setViewPosition(getPosition());
    m_view.show();
}

void StatusBarView::hide() { m_view.hide(); }

void StatusBarView::moveBy(int offsetX, int offsetY) {
    const QPoint offset(offsetX, offsetY);
    setViewPosition(m_view.position() + offset);
}

void StatusBarView::startDrag() {
    m_geometryManager.setDragStartMousePosition(QCursor::pos().x(),
                                                QCursor::pos().y());
}

void StatusBarView::endDrag() {
    setViewPosition(m_geometryManager.getValidPositionAfterDragEnd(
        QCursor::pos().x(), QCursor::pos().y()));
}

void StatusBarView::setViewPosition(QPoint viewPosition) {
    kdk::WindowManager::setGeometry(&m_view,
                                    QRect(viewPosition.x(), viewPosition.y(),
                                          m_view.width(), m_view.height()));
}

void StatusBarView::initConnections() {
    connect(&m_geometryManager, &GeometryManager::positionChanged,
            [this](QPoint pos) { setViewPosition(pos); });

    connect(&m_view, &QQuickView::widthChanged,
            [this]() { setViewPosition(getPosition()); });
}

void StatusBarView::initView(QPoint statusBarPos) {
    m_view.rootContext()->setContextProperty("statusBarModel", &m_model);
    m_view.rootContext()->setContextProperty("statusBarView", this);
    m_view.rootContext()->setContextProperty("statusBarManager", &m_manager);

    m_view.setColor(QColor(Qt::transparent));
    m_view.setFlags(Qt::Window | Qt::FramelessWindowHint |
                    Qt::WindowStaysOnTopHint | Qt::BypassWindowManagerHint);
    m_view.setSource(QUrl("qrc:/qml/StatusBar.qml"));
    setViewPosition(statusBarPos);
}

QPoint StatusBarView::getPosition() {
    return m_geometryManager.getValidPostionByInitialGeometry(
        m_view.geometry());
}
