/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "tooltip.h"
#include "windowmanager/windowmanager.h"

#include <QCursor>
#include <QGuiApplication>
#include <QQmlContext>
#include <QScreen>

ToolTip::ToolTip(QQuickItem *parent) : QQuickItem(parent) {
    initMouseEvent();
    initTimer();
    initView();
}

QString ToolTip::toolTipText() const { return m_toolTipText; }

void ToolTip::setToolTipText(const QString &text) {
    if (m_toolTipText != text) {
        m_toolTipText = text;
        emit toolTipTextChanged();
    }
}

void ToolTip::hideToolTip() {
    m_timer.stop();
    m_view.hide();
}

void ToolTip::showToolTip() {
    if (m_toolTipText.isEmpty()) {
        return;
    }
    setViewGeometry(QRect(getPosition().x(), getPosition().y(),
                          toolTipItem()->width(), toolTipItem()->height()));
    m_view.show();
    startTimeoutTimer();
}

void ToolTip::setViewGeometry(QRect viewGeometry) {
    kdk::WindowManager::setGeometry(&m_view, QRect(viewGeometry));
}

QQuickItem *ToolTip::toolTipItem() {
    QQuickItem *item = qobject_cast<QQuickItem *>(m_view.rootObject());
    item->setProperty("toolTipItem", QVariant::fromValue(this));
    return item;
}

void ToolTip::initMouseEvent() {
    setAcceptHoverEvents(true);
    setFiltersChildMouseEvents(true);
    setAcceptedMouseButtons(Qt::AllButtons);
}

void ToolTip::initTimer() { m_timer.setSingleShot(true); }

void ToolTip::initView() {
    m_view.setColor(QColor(Qt::transparent));
    m_view.setFlags(Qt::Window | Qt::FramelessWindowHint |
                    Qt::WindowStaysOnTopHint | Qt::BypassWindowManagerHint);
    m_view.setSource(QUrl("qrc:/qml/ToolTip.qml"));
}

void ToolTip::startTimeoutTimer() { startHideTimer(10000); }

void ToolTip::startShowTimer(int msec) {
    m_timer.disconnect();
    connect(&m_timer, &QTimer::timeout, [this]() { showToolTip(); });
    m_timer.start(msec);
}

void ToolTip::startHideTimer(int msec) {
    m_timer.disconnect();
    connect(&m_timer, &QTimer::timeout, [this]() { hideToolTip(); });
    m_timer.start(msec);
}

QPoint ToolTip::getPosition() {
    QRect currentScreenGeometry = getScreenGeometryOfPoint(QCursor::pos());
    return QPoint(getX(currentScreenGeometry.width(), toolTipItem()->width()),
                  getY(toolTipItem()->height()));
}

QRect ToolTip::getScreenGeometryOfPoint(QPoint point) {
    QScreen *currentScreen = qApp->screenAt(point);
    if (!currentScreen) {
        qWarning() << "The pointer is outside of any screen.";
        return QRect();
    }
    return currentScreen->geometry();
}

int ToolTip::getX(int screenWidth, int toolTipWidth) {
    // tooltip位置与按钮居中对齐
    int posX = parentItemX() + parentItem()->width() / 2 - toolTipWidth / 2;
    if (posX < 0) {
        return 0;
    }
    if (posX + toolTipWidth > screenWidth) {
        return screenWidth - toolTipWidth;
    }
    return posX;
}

int ToolTip::getY(int toolTipHeight) {
    int posY = parentItemY() - toolTipHeight - statusBarAndTooltipSpacing;
    int posYToolTipBottomStatusbar =
        parentItemY() + parentItem()->height() + statusBarAndTooltipSpacing;

    if (posY < 0) {
        return posYToolTipBottomStatusbar;
    }
    return posY;
}

int ToolTip::parentItemX() { return mapToGlobal(parentItem()->position()).x(); }

int ToolTip::parentItemY() { return mapToGlobal(parentItem()->position()).y(); }

void ToolTip::hoverEnterEvent(QHoverEvent *event) {
    Q_UNUSED(event);
    startShowTimer(400);
}

void ToolTip::hoverLeaveEvent(QHoverEvent *event) {
    Q_UNUSED(event);
    startHideTimer(500);
}

void ToolTip::mousePressEvent(QMouseEvent *event) {
    hideToolTip();
    event->setAccepted(false);
}
