/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "geometrymanager.h"

#include <QDebug>
#include <QGuiApplication>
#include <QScreen>

GeometryManager::GeometryManager(QObject *parent) : QObject(parent) {
    connectSignals();
}

void GeometryManager::setCursorPosition(const QRect &pos) { Q_UNUSED(pos) }

QPoint
GeometryManager::getValidPostionByInitialGeometry(const QRect &geometry) {
    m_geometry = geometry;
    updateValidPosition();
    updateMarginRatio();
    return m_geometry.topLeft();
}

void GeometryManager::setDragStartMousePosition(int globalMouseX,
                                                int globalMouseY) {
    m_dragStartX = globalMouseX;
    m_dragStartY = globalMouseY;
}

QPoint GeometryManager::getValidPositionAfterDragEnd(int globalMouseX,
                                                     int globalMouseY) {
    if (m_dragStartX < 0 || m_dragStartY < 0) {
        qWarning()
            << "Call setDragStartMousePosition() before calling this function.";
        return m_geometry.topLeft();
    }
    QRect currentScreenGeometry =
        getScreenGeometryOfPoint(QPoint(globalMouseX, globalMouseY));
    if (currentScreenGeometry.isNull()) {
        return m_geometry.topLeft();
    }

    // TODO：怎样拆分一下？只是把 X 和 Y
    // 坐标的调整代码放到单独的函数里好像并没有多大的意义。
    // 鼠标拖动窗口时，窗口始终处于鼠标所在的屏幕
    QRect newGeometry = m_geometry.translated(globalMouseX - m_dragStartX,
                                              globalMouseY - m_dragStartY);
    // X-coordinate
    if (newGeometry.left() < currentScreenGeometry.left()) {
        newGeometry.moveLeft(currentScreenGeometry.left());
    } else if (newGeometry.right() > currentScreenGeometry.right()) {
        newGeometry.moveRight(currentScreenGeometry.right());
    }
    // Y-coordinate
    if (newGeometry.top() < currentScreenGeometry.top()) {
        newGeometry.moveTop(currentScreenGeometry.top());
    } else if (newGeometry.bottom() > currentScreenGeometry.bottom()) {
        newGeometry.moveBottom(currentScreenGeometry.bottom());
    }
    m_geometry = newGeometry;
    updateMarginRatio();
    m_dragStartX = m_dragStartY = -1;
    return m_geometry.topLeft();
}

void GeometryManager::connectSignals() {
    connect(qApp->primaryScreen(), &QScreen::virtualGeometryChanged, this,
            &GeometryManager::onVirtualGeometryChanged);
    connect(qApp, &QGuiApplication::screenAdded, this,
            &GeometryManager::onVirtualGeometryChanged);
    connect(qApp, &QGuiApplication::screenRemoved, this,
            &GeometryManager::onVirtualGeometryChanged);
}

void GeometryManager::onVirtualGeometryChanged() {
    QSize screenSize = getScreenGeometryOfPoint(m_geometry.center()).size();
    if (screenSize.isNull()) {
        // 说明旋转后状态栏移除了屏幕，这时将状态栏移到主屏幕
        screenSize = qApp->primaryScreen()->availableGeometry().size();
    }
    float leftMargin =
        (screenSize.width() - m_geometry.width()) * m_leftMarginRatio;
    float topMargin =
        (screenSize.height() - m_geometry.height()) * m_topMarginRatio;
    m_geometry.moveTo(leftMargin, topMargin);
    qDebug() << __func__ << "and Signal from" << sender()
             << "NewGeometry: " << m_geometry;
    emit positionChanged(m_geometry.topLeft());
}

void GeometryManager::updateValidPosition() {
    QRect newGeometry = m_geometry;
    QRect currentScreenGeometry =
        getScreenGeometryOfPoint(m_geometry.topLeft());

    // X-coordinate
    if (newGeometry.right() > currentScreenGeometry.right() &&
        newGeometry.width() < currentScreenGeometry.width()) {
        newGeometry.moveRight(currentScreenGeometry.right());
    }
    // Y-coordinate
    if (newGeometry.bottom() > currentScreenGeometry.bottom() &&
        newGeometry.height() < currentScreenGeometry.height()) {
        newGeometry.moveBottom(currentScreenGeometry.bottom());
    }
    qDebug() << "Old Geometry: " << m_geometry
             << "New Geometry: " << newGeometry;
    m_geometry = newGeometry;
}

void GeometryManager::updateMarginRatio() {
    m_leftMarginRatio = calculateLeftMarginRatio(m_geometry.x());
    m_topMarginRatio = calculateTopMarginRatio(m_geometry.y());
}

float GeometryManager::calculateLeftMarginRatio(float leftMarginDistance) {
    return leftMarginDistance /
           (getScreenGeometryOfPoint(m_geometry.center()).width() -
            m_geometry.width());
}

float GeometryManager::calculateTopMarginRatio(float topMarginDistance) {
    return topMarginDistance /
           (getScreenGeometryOfPoint(m_geometry.center()).height() -
            m_geometry.height());
}

QRect GeometryManager::getScreenGeometryOfPoint(QPoint point) {
    QScreen *currentScreen = qApp->screenAt(point);
    if (!currentScreen) {
        qWarning() << "The pointer is outside of any screen.";
        return QRect();
    }
    return currentScreen->availableGeometry();
}
