/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef GEOMETRYMANAGER_H
#define GEOMETRYMANAGER_H

#include <QObject>
#include <QRect>

class GeometryManager : public QObject {
    Q_OBJECT
public:
    explicit GeometryManager(QObject *parent = nullptr);
    ~GeometryManager() = default;
    virtual void setCursorPosition(const QRect &pos);
    /**
     * @brief getValidPostionByInitialGeometry
     * 窗口初始化位置/输入窗口大小改变后，获取输入窗口的合法位置
     * @param geometry 输入窗口的当前位置
     * @return 输入窗口的合法位置
     */
    QPoint getValidPostionByInitialGeometry(const QRect &geometry);
    /**
     * @brief setDragStartMousePosition
     * 输入窗户口/状态栏位置开始移动前，设置鼠标的全局起始位置
     * @param globalMouseX 鼠标点击到窗口时的全局 X 坐标
     * @param globalMouseY 鼠标点击到窗口时的全局 Y 坐标
     */
    void setDragStartMousePosition(int globalMouseX, int globalMouseY);
    /**
     * @brief getValidPositionMoveWinBy
     * 输入窗户口/状态栏位置移动后，获取窗口的合法位置 调用该函数前必须先调用
     * setDragStartMousePosition 函数
     * @param globalMouseX 正在拖动窗口的鼠标的全局 X 坐标
     * @param globalMouseY 正在拖动窗口的鼠标的全局 Y 坐标
     * @return 窗口的合法位置
     */
    QPoint getValidPositionAfterDragEnd(int globalMouseX, int globalMouseY);
    float getLeftMarginRatio() const { return m_leftMarginRatio; }
    float getTopMarginRatio() const { return m_topMarginRatio; }

signals:
    void positionChanged(QPoint pos);

protected:
    virtual void updateValidPosition();
    QRect getScreenGeometryOfPoint(QPoint point);

private:
    void connectSignals();
    void onVirtualGeometryChanged();
    void updateMarginRatio();
    float calculateLeftMarginRatio(float leftMarginDistance);
    float calculateTopMarginRatio(float topMarginDistance);

protected:
    QRect m_cursorPosition;
    QRect m_geometry;

private:
    int m_dragStartX = -1;
    int m_dragStartY = -1;
    float m_leftMarginRatio = 0.0f;
    float m_topMarginRatio = 0.0f;
};

#endif // GEOMETRYMANAGER_H
