/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "statusbarsettings.h"

#include <QDir>
#include <QFile>
#include <QGuiApplication>
#include <QScreen>
#include <QSettings>

StatusBarSettings::StatusBarSettings() {
    if (homeConfigFileExists() == false) {
        createHomeConfigFile();
    }
    if (statusBarPosDataExists() == false) {
        initStatusBarDefaultPos();
    }
}

StatusBarSettings::StatusBarPositionData
StatusBarSettings::getStatusBarPos() const {
    QSettings settings(m_homeConfigFilePath, QSettings::NativeFormat);
    settings.beginGroup("StatusBarPosition");
    int globalX = settings.value("globalX").toInt();
    int globalY = settings.value("globalY").toInt();
    float regionalLeftRatio = settings.value("regionalLeftRatio").toFloat();
    float regionalTopRatio = settings.value("regionalTopRatio").toFloat();
    settings.endGroup();
    return StatusBarSettings::StatusBarPositionData{
        globalX, globalY, regionalLeftRatio, regionalTopRatio};
}

void StatusBarSettings::saveStatusBarPos(int globalX, int globalY) {
    QScreen *currentScreen = qApp->screenAt(QPoint(globalX, globalY));
    if (currentScreen == nullptr) {
        return;
    }
    QRect currentScreenGeo = currentScreen->availableGeometry();
    float regionalLeftRatio =
        static_cast<float>(globalX - currentScreenGeo.x()) /
        currentScreenGeo.width();
    float regionalTopRatio =
        static_cast<float>(globalY - currentScreenGeo.y()) /
        currentScreenGeo.height();
    saveStatusBarPositionData(StatusBarSettings::StatusBarPositionData{
        globalX, globalY, regionalLeftRatio, regionalTopRatio});
}

bool StatusBarSettings::homeConfigFileExists() const {
    return QFile::exists(m_homeConfigFilePath);
}

bool StatusBarSettings::createHomeConfigFile() {
    if (QFile::exists(m_configFilePath) == false) {
        return false;
    }
    QDir dir(m_homeConfigFileDir);
    if (dir.exists() == false) {
        dir.mkpath(m_homeConfigFileDir);
    }
    return QFile::copy(m_configFilePath, m_homeConfigFilePath);
}

bool StatusBarSettings::statusBarPosDataExists() const {
    bool globalXExists = false;
    bool globalYExists = false;
    QSettings settings(m_homeConfigFilePath, QSettings::NativeFormat);
    settings.beginGroup("StatusBarPosition");
    settings.value("globalX").toInt(&globalXExists);
    settings.value("globalY").toInt(&globalYExists);
    settings.endGroup();
    return globalXExists && globalYExists;
}

void StatusBarSettings::initStatusBarDefaultPos() {
    QRect screenGeo = qApp->primaryScreen()->availableGeometry();
    const float defaultRatio = 5.0f / 6;
    int globalX = screenGeo.x() + screenGeo.width() * defaultRatio;
    int globalY = screenGeo.y() + screenGeo.height() * defaultRatio;
    saveStatusBarPositionData(StatusBarSettings::StatusBarPositionData{
        globalX, globalY, defaultRatio, defaultRatio});
}

void StatusBarSettings::saveStatusBarPositionData(
    const StatusBarSettings::StatusBarPositionData &pos) {
    QSettings settings(m_homeConfigFilePath, QSettings::NativeFormat);
    settings.beginGroup("StatusBarPosition");
    settings.setValue("globalX", pos.globalX);
    settings.setValue("globalY", pos.globalY);
    settings.setValue("regionalLeftRatio", pos.regionalLeftRatio);
    settings.setValue("regionalTopRatio", pos.regionalTopRatio);
    settings.endGroup();
}